/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.java.net
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.integritychecker.impl;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.ThreadSafe;
import java.io.IOException;
import java.io.File;
import java.io.RandomAccessFile;
import java.nio.MappedByteBuffer;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import it.tidalwave.actor.annotation.Actor;
import it.tidalwave.actor.annotation.MessageListener;
import it.tidalwave.integritychecker.FileDamageDetectedMessage;
import it.tidalwave.integritychecker.FileDiscoveredMessage;
import lombok.Cleanup;
import lombok.extern.slf4j.Slf4j;
import static java.nio.channels.FileChannel.MapMode.READ_ONLY;

/***********************************************************************************************************************
 * 
 * @stereotype Actor
 * 
 * An actor for computing the fingerprint of a file.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Actor(initialPriority=Thread.MAX_PRIORITY) @ThreadSafe @Slf4j
public class FileLoaderActor
  {
    private long previousTime = System.currentTimeMillis();
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @MessageListener
    public void process (final @Nonnull FileDiscoveredMessage message) 
      {
        final FileObject fileObject = message.getFileObject();

        try 
          {
            final long startTime = System.currentTimeMillis();
            final File file = FileUtil.toFile(fileObject);
            final @Cleanup RandomAccessFile randomAccessFile = new RandomAccessFile(file, "r");
            final MappedByteBuffer byteBuffer = randomAccessFile.getChannel().map(READ_ONLY, 0, file.length());  
            byteBuffer.load();
            randomAccessFile.close();
            final long busyTime = startTime - previousTime;
            previousTime = System.currentTimeMillis();
            final long time = previousTime - startTime;
//            log.info(">>>> load speed: {} MB/s - was busy for {} msec", byteBuffer.limit() / (time * 1000), busyTime);
            new FileContentsAvailableMessage(fileObject, byteBuffer, time).send();
          }
        catch (IOException e) 
          {
            FileDamageDetectedMessage.forFile(fileObject).withCause(e).send();
          }
        catch (Exception e) 
          {
            FileDamageDetectedMessage.forFile(fileObject).withCause(e).send();
          }
      }
  }
